% 2-D beam propagator, FFT based method
% physics = vectorial field in a wide-angle beam

% parameters 
lambda = 633.0e-09;          
w0     = 10*lambda;

zR     = pi*w0^2/lambda;
distance = 1.0*zR;

LX     = 20.0*w0;
NX     = 512;
dz     = distance/150.0;
LZ     = distance;


% derived parameters
k0 = 2*pi/lambda;
dk = 2*pi/LX;
dx = LX/NX;
stps = LZ/dz;

t = cputime;

% coordinates
cx = dx*(linspace(0,NX-1,NX)-NX/2); 

% transverse wavenumbers
kx = zeros(1,NX);
for k=0:NX/2
	kx(1+k) = dk*k;
end
for k=NX/2+1:NX-1
	kx(1+k) = dk*(k - NX);
end

% amplitude holder, define an initial condition
am0 = zeros(NX,NX);
for x=1:NX
  for y=1:NX
  am0(x,y) = GaussianBeam1DRotated(cx(x) - cx(NX)/2,-distance,w0,k0,0.25)*GaussianBeam1DRotated(cx(y),-distance,w0,k0,0.0);
  end
end

% propagator
pxy = zeros(NX,NX);
for x=1:NX
  for y=1:NX
	  pxy(x,y) = exp(+1i*dz*sqrt(k0^2 - (kx(x)^2 + kx(y)^2)));
  end
end


fprintf(1,'CPU time in initialization: %g\n',cputime - t);

% define one linear step
LinearStep = @(amplitudein,propagator) ifft2(propagator.*fft2(amplitudein));

t = cputime;
tic;

% execute one step
am1 = am0;

for s=1:stps
  fprintf(1,'%d out of %d , distance = %d\n',s,stps,s*dz);
  am1 = LinearStep(am1,pxy);
  % plot result

  figure(1);
  hold off;
  plot(cx,real(am1(1:NX,NX/2)),'b');
  ylim([-1.1 1.1]);
  title('real Ex');
  pause(0.01);


end

fprintf(1,'CPU time in steps: %g\n',cputime - t);
fprintf(1,'elapsed time in steps: %g\n',toc);

figure(2)
clf;
subplot(4,1,1)
plot(kx,real(pxy(1:NX,1)));
title('Propagator');
subplot(4,1,2)
plot(kx,abs(fft(am0(1:NX,NX/4))));
title('Initial spatial spectrum (abs)');
subplot(4,1,3)
plot(cx,abs(am0(1:NX,NX/2)));
title('Initial condition (abs)');
subplot(4,1,4)
hold on;
plot(cx,abs(am1(1:NX,NX/2)),'b');
title('Propagated complex amplitude (abs)');
hold off;
title('diagnostic auxiliaries');

figure(3);
imagesc(real(am1));
colorbar;
title('X-component, electric field');

% calculate Ez from the div.E=0 equation, assuming Ey=0
spatial = fft2(am1);
operate = zeros(NX,NX);
for x=1:NX
    for y=1:NX
        if( k0^2 > kx(x)^2 + kx(y)^2 )
         operate(x,y) = -kx(x)/sqrt(k0^2 - kx(x)^2 - kx(y)^2); 
        end
    end
end
am2 = ifft2(spatial.*operate);

figure(4);
imagesc(real(am2));
colorbar;
title('Z-component electric field');

